#ifndef LOG4QT_LOGGER_H
#define LOG4QT_LOGGER_H

#include <QtCore/QObject>

#include <QtCore/QAtomicPointer>
#include <QtCore/QList>
#include <QtCore/QReadWriteLock>
#include "helpers/logerror.h"
#include "helpers/classlogger.h"
#include "helpers/logobjectptr.h"
#include "level.h"

#if QT_VERSION >= QT_VERSION_CHECK(4, 4, 0)
#	ifndef Q_ATOMIC_POINTER_TEST_AND_SET_IS_ALWAYS_NATIVE
#		warning "QAtomicPointer test and set is not native. The macro Log4Qt::LOG4QT_DECLARE_STATIC_LOGGER is not thread-safe."
#	endif
#endif

typedef void(*ErrorCallback)();

namespace Log4Qt
{
	/*!
	 * LOG4QT_DECLARE_STATIC_LOGGER declares a static function \a FUNCTION that
	 * returns a pointer to a \ref Log4Qt::Logger "Logger" named after \a CLASS.
	 *
	 * On the first invocation the \ref Log4Qt::Logger "Logger" is requested
	 * by calling \ref Log4Qt::Logger::logger(const char *pName)
	 * "Logger::logger( #CLASS )". The pointer is stored to be returned on
	 * subsequent invocations.
	 *
	 * The following example shows how to use the macro to define a logger to be
	 * used within a class not derived from QObject.
	 *
	 * \code
	 * #file: counter.h
	 *
	 * #include logger.h
	 *
	 * class Counter
	 * {
	 * public:
	 *     Counter();
	 *     Counter(int preset);
	 * private:
	 *     int mCount;
	 * }
	 * \endcode
	 * \code
	 * #file: counter.cpp
	 *
	 * #include counter.h
	 *
	 * LOG4QT_DECLARE_STATIC_LOGGER(logger, Counter)
	 *
	 * Counter::Counter() :
	 *     mCount(0)
	 * {}
	 *
	 * void Counter::Counter(int preset) :
	 *     mCount(preset)
	 * {
	 *     if (preset < 0)
	 *     {
	 *         logger()->warn("Invalid negative counter preset %1. Using 0 instead.", preset);
	 *         mCount = 0;
	 *     }
	 * }
	 * \endcode
	 *
	 * \note The function created by the macro is thread-safe.
	 *
	 * \sa \ref Log4Qt::Logger::logger(const char *pName) "Logger::logger(const char *pName)"
	 */
#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
    #define LOG4QT_DECLARE_STATIC_LOGGER(FUNCTION, CLASS)                     \
        static Log4Qt::Logger *FUNCTION()                                     \
        {                                                                     \
            static QBasicAtomicPointer<Log4Qt::Logger > p_logger =            \
                Q_BASIC_ATOMIC_INITIALIZER(0);                                \
            if (!p_logger)                                                    \
            {                                                                 \
                p_logger.testAndSetOrdered(0,                                 \
                    Log4Qt::Logger::logger( #CLASS ));                        \
            }                                                                 \
            return p_logger;                                                  \
        }
#else
	#define LOG4QT_DECLARE_STATIC_LOGGER(FUNCTION, CLASS)                     \
	    static Log4Qt::Logger *FUNCTION()                                     \
	    {                                                                     \
			static QBasicAtomicPointer<Log4Qt::Logger > p_logger =            \
				Q_BASIC_ATOMIC_INITIALIZER(0);                                \
            if (!p_logger.loadAcquire())                                      \
	        {                                                                 \
	        	p_logger.testAndSetOrdered(0,                                 \
					Log4Qt::Logger::logger( #CLASS ));                        \
            }                                                                 \
            return p_logger.loadAcquire();                                    \
	    }
#endif

	/*!
	 * LOG4QT_DECLARE_QCLASS_LOGGER declares member functions to retrieve
	 * \ref Log4Qt::Logger "Logger" for the class it is used in.
	 *
	 * On the first invocation the \ref Log4Qt::Logger "Logger" is requested
	 * by a call to \ref Log4Qt::Logger::logger(const char *pName)
	 * "Logger::logger(const char *pName)". The pointer is stored to be
	 * returned on subsequent invocations.
	 *
	 * The following example shows how to use the macro to define a logger to be
	 * used within a class derived from QObject.
	 *
	 * \code
	 * #file: counter.h
	 *
	 * #include qobject.h
	 * #include logger.h
	 *
	 * class Counter : public QObject
	 * {
	 *     Q_OBJECT
	 *     LOG4QT_DECLARE_QCLASS_LOGGER
	 * public:
	 *     Counter();
	 *     Counter(int preset);
	 * private:
	 *     int mCount;
	 * }
	 * \endcode
	 * \code
	 * #file: counter.cpp
	 *
	 * #include counter.h
	 *
	 * Counter::Counter() :
	 *     mCount(0)
	 * {}
	 *
	 * void Counter::Counter(int preset)
	 *     mCount(preset)
	 * {
	 *     if (preset < 0)
	 *     {
	 *         logger()->warn("Invalid negative counter preset %1. Using 0 instead.", preset);
	 *         mCount = 0;
	 *     }
	 * }
	 * \endcode
	 *
	 * \note The function created by the macro is thread-safe.
	 *
	 * \sa \ref Log4Qt::Logger::logger(const char *pName) "Logger::logger(const char *pName)",
	 *     \ref Log4Qt::ClassLogger "ClassLogger"
	 */
	#define LOG4QT_DECLARE_QCLASS_LOGGER                                      \
	    private:                                                              \
	        mutable Log4Qt::ClassLogger mLog4QtClassLogger;                   \
	    public:                                                               \
            Log4Qt::Logger *logger() const                                    \
	        {   return mLog4QtClassLogger.logger(this);    }                  \
	    private:

	class Appender;
	class LoggingEvent;
	class LoggerRepository;

	/*!
	 * \brief The class Logger provides logging services.
	 *
	 * A pointer to a logger can be retrieved by calling Logger::logger() or
	 * LogManager::logger() with the class name as argument. Because a logger
	 * is never destroyed it is possible to store the pointer to the logger.
	 * This way the lookup of the pointer in the repository is only required
	 * on the first logging operation. The macros \ref
	 * Log4Qt::LOG4QT_DECLARE_STATIC_LOGGER "LOG4QT_DECLARE_STATIC_LOGGER" and
	 * \ref Log4Qt::LOG4QT_DECLARE_QCLASS_LOGGER "LOG4QT_DECLARE_QCLASS_LOGGER"
	 * provide a thread-safe implementation to store the logger pointer.
	 *
	 * \note All the functions declared in this class are thread-safe.
	 */
	class Logger : public QObject
	{
	    Q_OBJECT

	    /*!
	     * The property holds, if the logger is additive.
	     *
	     * The default is true for being additive.
	     *
	     * \sa additive(), setAdditive()
	     */
	    Q_PROPERTY(bool additivity READ additivity WRITE setAdditivity)

	    /*!
	     * The property holds the level used by the logger.
	     *
	     * The default is Level::NULL_INT.
	     * \sa level(), setLevel()
	     */
	    Q_PROPERTY(Level level READ level WRITE setLevel)

	    /*!
	     * The property holds the LoggerRepository of the logger.
	     *
	     * \sa loggerRepository()
	     */
	    Q_PROPERTY(LoggerRepository* loggerRepository READ loggerRepository)

	    /*!
	     * The property holds the name of the logger.
	     *
	     * \sa name()
	     */
	    Q_PROPERTY(QString name READ name)

	    /*!
	     * The property holds the parent logger of the logger.
	     *
         * \sa parentLogger()
	     */
	    Q_PROPERTY(Logger* parentLogger READ parentLogger)

	    LOG4QT_DECLARE_QCLASS_LOGGER

	protected:
        Logger(LoggerRepository* pLoggerRepository, Level level, const QString &rName, const QString &alias, Logger *pParent = 0);
	    virtual ~Logger();

	private:
	    Logger(const Logger &rOther); // Not implemented
	    Logger &operator=(const Logger &rOther); // Not implemented

	public:
	    bool additivity() const;
	    QList<Appender*> appenders() const;
	    Level level() const;
	    LoggerRepository *loggerRepository() const;
	    QString name() const;
	    
	    /**
	     * @brief Установка и получение псевдонима логгера
	     */
        void setAlias(const QString &value);
        QString getAlias() const;
	    Logger *parentLogger() const;

	    void setAdditivity(bool additivity);
	    virtual void setLevel(Level level);

	    void addAppender(Appender *pAppender);
	    Appender *appender(const QString &rName) const;
	    void callAppenders(const LoggingEvent &rEvent) const;
	    bool isAttached(Appender *pAppender) const;

	    /*!
	     * Removes all appenders that have been previously added from this
	     * Logger.
	     *
	     * To allow configurators to collect events during the configuration
	     * process ListAppenders with the configuratorList property set, will
	     * not be removed.
	     *
	     * \sa LisAppender::setConfiguratorList()
	     */
	    void removeAllAppenders();

	    void removeAppender(Appender *pAppender);
	    void removeAppender(const QString &rName);

	    Level effectiveLevel() const;
	    bool isDebugEnabled() const;

		/*!
		 * Checks if this logger is enabled for a given Level \a level. If the
		 * logger is enabled the function returns true. Otherwise it returns
		 * false.
		 *
		 * A logger is enabled for a level, if the level is greater or equal
		 * then the repository threshold and greater and equal then the loggers
		 * effective level.
		 *
		 * \sa LoggerRepository::isDisabled(), effectiveLevel()
		 */
	    bool isEnabledFor(Level level) const;

	    bool isErrorEnabled() const;
	    bool isFatalEnabled() const;
	    bool isInfoEnabled() const;
	    bool isTraceEnabled() const;
	    bool isWarnEnabled() const;

	    void debug(const QString &rMessage) const;
	    void debug(const LogError &rLogError) const;
	    void debug(const char *pMessage) const;
        void debug(const char *pMessage, const QString &rArg1) const;
        void debug(const char *pMessage, int arg1) const;
        void debug(const char *pMessage, const QString &rArg1, const QString &rArg2) const;
        void debug(const char *pMessage, const QString &rArg1, int arg2) const;
        void debug(const char *pMessage, int arg1, const QString &rArg2) const;
        void debug(const char *pMessage, int arg1, int arg2) const;
        void debug(const char *pMessage, const QString &rArg1, const QString &rArg2, const QString &rArg3) const;
        void debug(const char *pMessage, const QString &rArg1, const QString &rArg2, int arg3) const;
        void debug(const char *pMessage, const QString &rArg1, int arg2, const QString &rArg3) const;
        void debug(const char *pMessage, const QString &rArg1, int arg2, int arg3) const;
        void debug(const char *pMessage, int arg1, const QString &rArg2, const QString &rArg3) const;
        void debug(const char *pMessage, int arg1, const QString &rArg2, int arg3) const;
        void debug(const char *pMessage, int arg1, int arg2, const QString &rArg3) const;
        void debug(const char *pMessage, int arg1, int arg2, int arg3) const;
        void debug(const char *pMessage, const QVariant &rArg1, const QVariant &rArg2, const QVariant &rArg3) const;

	    void error(const QString &rMessage) const;
	    void error(const LogError &rLogError) const;
	    void error(const char *pMessage) const;
        void error(const char *pMessage, const QString &rArg1) const;
        void error(const char *pMessage, int arg1) const;
        void error(const char *pMessage, const QString &rArg1, const QString &rArg2) const;
        void error(const char *pMessage, const QString &rArg1, int arg2) const;
        void error(const char *pMessage, int arg1, const QString &rArg2) const;
        void error(const char *pMessage, int arg1, int arg2) const;
        void error(const char *pMessage, const QString &rArg1, const QString &rArg2, const QString &rArg3) const;
        void error(const char *pMessage, const QString &rArg1, const QString &rArg2, int arg3) const;
        void error(const char *pMessage, const QString &rArg1, int arg2, const QString &rArg3) const;
        void error(const char *pMessage, const QString &rArg1, int arg2, int arg3) const;
        void error(const char *pMessage, int arg1, const QString &rArg2, const QString &rArg3) const;
        void error(const char *pMessage, int arg1, const QString &rArg2, int arg3) const;
        void error(const char *pMessage, int arg1, int arg2, const QString &rArg3) const;
        void error(const char *pMessage, int arg1, int arg2, int arg3) const;
        void error(const char *pMessage, const QVariant &rArg1, const QVariant &rArg2, const QVariant &rArg3) const;

	    void fatal(const QString &rMessage) const;
	    void fatal(const LogError &rLogError) const;
	    void fatal(const char *pMessage) const;
        void fatal(const char *pMessage, const QString &rArg1) const;
        void fatal(const char *pMessage, int arg1) const;
        void fatal(const char *pMessage, const QString &rArg1, const QString &rArg2) const;
        void fatal(const char *pMessage, const QString &rArg1, int arg2) const;
        void fatal(const char *pMessage, int arg1, const QString &rArg2) const;
        void fatal(const char *pMessage, int arg1, int arg2) const;
        void fatal(const char *pMessage, const QString &rArg1, const QString &rArg2, const QString &rArg3) const;
        void fatal(const char *pMessage, const QString &rArg1, const QString &rArg2, int arg3) const;
        void fatal(const char *pMessage, const QString &rArg1, int arg2, const QString &rArg3) const;
        void fatal(const char *pMessage, const QString &rArg1, int arg2, int arg3) const;
        void fatal(const char *pMessage, int arg1, const QString &rArg2, const QString &rArg3) const;
        void fatal(const char *pMessage, int arg1, const QString &rArg2, int arg3) const;
        void fatal(const char *pMessage, int arg1, int arg2, const QString &rArg3) const;
        void fatal(const char *pMessage, int arg1, int arg2, int arg3) const;
        void fatal(const char *pMessage, const QVariant &rArg1, const QVariant &rArg2, const QVariant &rArg3) const;

	    void info(const QString &rMessage) const;
	    void info(const LogError &rLogError) const;
	    void info(const char *pMessage) const;
        void info(const char *pMessage, const QString &rArg1) const;
        void info(const char *pMessage, int arg1) const;
        void info(const char *pMessage, const QString &rArg1, const QString &rArg2) const;
        void info(const char *pMessage, const QString &rArg1, int arg2) const;
        void info(const char *pMessage, int arg1, const QString &rArg2) const;
        void info(const char *pMessage, int arg1, int arg2) const;
        void info(const char *pMessage, const QString &rArg1, const QString &rArg2, const QString &rArg3) const;
        void info(const char *pMessage, const QString &rArg1, const QString &rArg2, int arg3) const;
        void info(const char *pMessage, const QString &rArg1, int arg2, const QString &rArg3) const;
        void info(const char *pMessage, const QString &rArg1, int arg2, int arg3) const;
        void info(const char *pMessage, int arg1, const QString &rArg2, const QString &rArg3) const;
        void info(const char *pMessage, int arg1, const QString &rArg2, int arg3) const;
        void info(const char *pMessage, int arg1, int arg2, const QString &rArg3) const;
        void info(const char *pMessage, int arg1, int arg2, int arg3) const;
        void info(const char *pMessage, const QVariant &rArg1, const QVariant &rArg2, const QVariant &rArg3) const;

        void log(Level level, const QString &rMessage) const;
        void log(Level level, const LogError &rLogError) const;
        void log(Level level, const char *pMessage) const;
        void log(Level level, const char *pMessage, const QString &rArg1) const;
        void log(Level level, const char *pMessage, int arg1) const;
        void log(Level level, const char *pMessage, const QString &rArg1, const QString &rArg2) const;
        void log(Level level, const char *pMessage, const QString &rArg1, int arg2) const;
        void log(Level level, const char *pMessage, int arg1, const QString &rArg2) const;
        void log(Level level, const char *pMessage, int arg1, int arg2) const;
        void log(Level level, const char *pMessage, const QString &rArg1,
                const QString &rArg2, const QString &rArg3) const;
        void log(Level level, const char *pMessage, const QString &rArg1, const QString &rArg2, int arg3) const;
        void log(Level level, const char *pMessage, const QString &rArg1, int arg2, const QString &rArg3) const;
        void log(Level level, const char *pMessage, const QString &rArg1, int arg2, int arg3) const;
        void log(Level level, const char *pMessage, int arg1, const QString &rArg2, const QString &rArg3) const;
        void log(Level level, const char *pMessage, int arg1, const QString &rArg2, int arg3) const;
        void log(Level level, const char *pMessage, int arg1, int arg2, const QString &rArg3) const;
        void log(Level level, const char *pMessage, int arg1, int arg2, int arg3) const;
        void log(Level level, const char *pMessage, const QVariant &rArg1,
                const QVariant &rArg2, const QVariant &rArg3) const;

	    void trace(const QString &rMessage) const;
	    void trace(const LogError &rLogError) const;
	    void trace(const char *pMessage) const;
        void trace(const char *pMessage, const QString &rArg1) const;
        void trace(const char *pMessage, int arg1) const;
        void trace(const char *pMessage, const QString &rArg1, const QString &rArg2) const;
        void trace(const char *pMessage, const QString &rArg1, int arg2) const;
        void trace(const char *pMessage, int arg1, const QString &rArg2) const;
        void trace(const char *pMessage, int arg1, int arg2) const;
        void trace(const char *pMessage, const QString &rArg1, const QString &rArg2, const QString &rArg3) const;
        void trace(const char *pMessage, const QString &rArg1, const QString &rArg2, int arg3) const;
        void trace(const char *pMessage, const QString &rArg1, int arg2, const QString &rArg3) const;
        void trace(const char *pMessage, const QString &rArg1, int arg2, int arg3) const;
        void trace(const char *pMessage, int arg1, const QString &rArg2, const QString &rArg3) const;
        void trace(const char *pMessage, int arg1, const QString &rArg2, int arg3) const;
        void trace(const char *pMessage, int arg1, int arg2, const QString &rArg3) const;
        void trace(const char *pMessage, int arg1, int arg2, int arg3) const;
        void trace(const char *pMessage, const QVariant &rArg1, const QVariant &rArg2, const QVariant &rArg3) const;

	    void warn(const QString &rMessage) const;
	    void warn(const LogError &rLogError) const;
	    void warn(const char *pMessage) const;
        void warn(const char *pMessage, const QString &rArg1) const;
        void warn(const char *pMessage, int arg1) const;
        void warn(const char *pMessage, const QString &rArg1, const QString &rArg2) const;
        void warn(const char *pMessage, const QString &rArg1, int arg2) const;
        void warn(const char *pMessage, int arg1, const QString &rArg2) const;
        void warn(const char *pMessage, int arg1, int arg2) const;
        void warn(const char *pMessage, const QString &rArg1, const QString &rArg2, const QString &rArg3) const;
        void warn(const char *pMessage, const QString &rArg1, const QString &rArg2, int arg3) const;
        void warn(const char *pMessage, const QString &rArg1, int arg2, const QString &rArg3) const;
        void warn(const char *pMessage, const QString &rArg1, int arg2, int arg3) const;
        void warn(const char *pMessage, int arg1, const QString &rArg2, const QString &rArg3) const;
        void warn(const char *pMessage, int arg1, const QString &rArg2, int arg3) const;
        void warn(const char *pMessage, int arg1, int arg2, const QString &rArg3) const;
        void warn(const char *pMessage, int arg1, int arg2, int arg3) const;
        void warn(const char *pMessage, const QVariant &rArg1, const QVariant &rArg2, const QVariant &rArg3) const;

	    // LogManager operations
	    static Logger *logger(const QString &rName);
	    static Logger *logger(const char *pName);
	    static Logger *rootLogger();

        /**
         * @brief Установка callback-функции при возникновении ошибки записи
         * @param function - функция
         */
        static void setErrorCallback(ErrorCallback function);

        /**
         * @brief Вызов callback-функции
         */
        static void callErrorCallback();
	protected:
#ifndef QT_NO_DEBUG_STREAM
	    /*!
	     * Writes all object member variables to the given debug stream \a rDebug
	     * and returns the stream.
	     *
	     * <tt>
	     * %Logger(name:"Log4Qt" appenders:0 additivity:true Level("NULL")
	     *         parentLogger: "root" )
	     * </tt>
	     * \sa QDebug, operator<<(QDebug debug, const Appender &rAppender)
	     */
		virtual QDebug debug(QDebug &rDebug) const;
        friend QDebug operator<<(QDebug debug, const Logger &rLogger);
#endif // QT_NO_DEBUG_STREAM

        void forcedLog(Level level, const QString &rMessage) const;

	protected:
	    mutable QReadWriteLock mObjectGuard;

	private:
        QString mName;
        QString alias;
	    LoggerRepository* mpLoggerRepository;
	    volatile bool mAdditivity;
	    QList< LogObjectPtr<Appender> > mAppenders;
	    Level mLevel;
	    Logger *mpParent;
        static ErrorCallback errorCallback;         /**< callback-функция, вызываемая при возникновении ошибки записи */
	    // Needs to be friend to create Logger objects
	    friend class Hierarchy;
	};

#ifndef QT_NO_DEBUG_STREAM
	/*!
	 * \relates Logger
	 *
	 * Writes all object member variables to the given debug stream \a debug and
	 * returns the stream.
	 *
	 * To handle subclassing the function uses the virtual member function debug().
	 * This allows each class to generate its own output.
	 *
	 * \sa QDebug, debug()
	 */
    QDebug operator<<(QDebug debug, const Logger &rLogger);
#endif // QT_NO_DEBUG_STREAM

} // namespace Log4Qt

// Q_DECLARE_TYPEinfo(Log4Qt::Logger, Q_COMPLEX_TYPE); // Use default

#endif // LOG4QT_LOGGER_H
